﻿Imports System.Globalization
Imports System.Windows.Markup
Imports MahApps.Metro
Imports System.Windows.Shell
Imports System.Threading
Imports System.Windows.Threading
Imports System.Windows
Imports System.Linq
Imports System

Class Application

    Protected Overrides Sub OnStartup(e As StartupEventArgs)
        MyBase.OnStartup(e)
        StartSplash()
        TrySplashMessage("HHHB3 wird gestartet...")

        If check_firstinstallation_themesettings() Then
            cls_logger.v_logger.Info("First Installation Theme Settings have been written to registry")
        Else
            cls_logger.v_logger.Error("First Installation Theme Settings write failed!")
            MessageBox.Show("Theme Settings konnten nicht erstellt werden, Programm wird beendet", "Fehler", MessageBoxButton.OK, MessageBoxImage.Stop)
            Environment.Exit(0)
        End If

#If Not DEBUG Then
        'Auf Updates prüfen:
        Dim __updates As New cls_updater
        If __updates.check_for_updates = True Then
            TrySplashMessage("Updates werden gesucht...")
            __updates.do_update()
        End If
#End If

        'Kultur setzen
        TrySplashMessage("Kultur setzen")
        Dim current As CultureInfo = CultureInfo.CurrentUICulture
        Dim newUICulture As CultureInfo
        If Not current.Name.Equals("de-AT") Orelse current.Name.Equals("de-DE") Then
            newUICulture = New CultureInfo("de-DE")
            CultureInfo.DefaultThreadCurrentUICulture = newUICulture
            cls_logger.v_logger.Info("Kultur gesetzt")
        End If
        FrameworkElement.LanguageProperty.OverrideMetadata(GetType(FrameworkElement), New FrameworkPropertyMetadata(XmlLanguage.GetLanguage(CultureInfo.CurrentCulture.IetfLanguageTag)))

        TrySplashMessage("Theme setzen")
        Dim v_metrotheme As String = GetValueFromRegistryKey("metro_theme")
        Dim v_metroaccent As String = GetValueFromRegistryKey("metro_accent")

        'Theme auf die letzte Einstellung setzen:
        ThemeManager.ChangeAppStyle(Application.Current, ThemeManager.GetAccent(v_metroaccent), ThemeManager.GetAppTheme(v_metrotheme))
        'same for the Fluent Ribbon
        Fluent.ThemeManager.ChangeAppStyle(Application.Current, Fluent.ThemeManager.GetAccent(v_metroaccent), Fluent.ThemeManager.GetAppTheme(v_metrotheme))
        v_metrotheme = Nothing
        v_metroaccent = Nothing
        'Add the custom theme:
        'Deactivated until the theme for the ribbon is finished.
        'ThemeManager.AddAppTheme("BaseBlue", New Uri("pack://application:,,,/HHHB3;component/Themes/Theme/BaseBlue.xaml"))
        'Fluent.ThemeManager.AddAppTheme("BaseBlue", New Uri("pack://application:,,,/HHHB3;component/Themes/Theme/BaseBlue.xaml"))

        Dim v_lastused_db_path As String = GetValueFromRegistryKey("db_lastused")

        '1. step check if any arguments are passed through:
        cls_logger.v_logger.Info("Startup dblastused: " & v_lastused_db_path)
        'if any arguments passed with the startup change db lastused
        If e.Args.Count > 0 Then
            If e.Args(0) = "-isrestart" Then
                isRestart = True
            Else
                isRestart = False
                Debug.Print("Argument: " & e.Args(0).ToString)
                'MessageBox.Show("Argument: " & e.Args(0).ToString)
                If Not v_lastused_db_path.Equals(e.Args(0).ToString) Then
                    'Change Database Connectionstring:
                    SetValueFromRegistryKey("db_lastused", e.Args(0).ToString)
                    cls_logger.v_logger.Info("DB ausgetauscht, neu starten: " & e.Args(0).ToString)
                    ShutdownOrRestartApp(True)
                End If
            End If
        End If

        TrySplashMessage("Datenbank wird geprüft...")
        cls_logger.v_logger.Info("Datenbank wird geprüft")

        If String.IsNullOrEmpty(v_lastused_db_path) Then
            'There is no lastused db start the dialog:
            If AskDBPath() = False Then
                'Seems that the user aborted the dialog so kill the application:
                ShutdownOrRestartApp(False)
            Else
                ShutdownOrRestartApp(True)
            End If
        End If

        'Set the database :
        If set_db() = set_db_results.set_new_restart Then
            'Connectionstring has changed restart:
            cls_logger.v_logger.Info("set_db returned with set_new_restart perform restart")
            ShutdownOrRestartApp(True)
        End If


        TrySplashMessage("Daten laden...")
        Dim w = New MainWindow()
        AddHandler w.ContentRendered, Sub(s, cre) _Splash.Dispatcher.BeginInvoke(DirectCast(AddressOf CleanUpSplash, Action))
        MainWindow = w
        w.ShowActivated = True
        w.Show()
        w.Activate()
        w.BringIntoView()
    End Sub

    ''' <summary>
    ''' Function to check if HHHB3 is first installed to a machine
    ''' Defines the presets for theme, color and icons in the registry
    ''' </summary>
    ''' <returns></returns>
    Public Function check_firstinstallation_themesettings() As Boolean
        Try
            'used_icons
            If String.IsNullOrEmpty(GetValueFromRegistryKey("used_icons")) Then
                'Set the preset value:
                SetValueFromRegistryKey("used_icons", "win10")
                cls_logger.v_logger.Info("used_icons preset set in registry")
            End If
            'metro_accent
            If String.IsNullOrEmpty(GetValueFromRegistryKey("metro_accent")) Then
                'Set the preset value:
                SetValueFromRegistryKey("metro_accent", "Blue")
                cls_logger.v_logger.Info("metro_accent preset set in registry")
            End If
            'metro_theme
            If String.IsNullOrEmpty(GetValueFromRegistryKey("metro_theme")) Then
                'Set the preset value:
                SetValueFromRegistryKey("metro_theme", "BaseLight")
                cls_logger.v_logger.Info("metro_theme preset set in registry")
            End If
        Catch ex As Exception
            cls_logger.v_logger.Error("check_firstinstallation_themesettings failed with: " & ex.Message)
            Return False
        End Try
        Return True
    End Function


    'splashmessage wurde in ein Globales Modul mod_help gepackt!
    Private Sub StartSplash()
        Dim thr = New Thread(Sub()
                                 _Splash = New splash()
                                 _Splash.ShowDialog()
                             End Sub) With {
            .Name = "hhhb3_splash",
            .Priority = ThreadPriority.Highest
                             }
        thr.SetApartmentState(ApartmentState.STA)
        thr.Start()
    End Sub

    'Sub for creating a dialog to ask if the user want's to create a new database or use an existing one:
    Private Function AskDBPath() As Boolean
        Dim v_start_dialog As New frm_start
        With v_start_dialog
            .ShowDialog()
            If .DialogResult Then
                SetValueFromRegistryKey("db_lastused", .db_path)
                Return True
            Else
                Return False
            End If
        End With
    End Function

    Private Sub ShutdownOrRestartApp(v_restart As Boolean)
        Try
            _Splash.Dispatcher.BeginInvoke(DirectCast(AddressOf CleanUpSplash, Action))
        Catch ex As Exception
            cls_logger.v_logger.Error(String.Format("ShutdownOrRestartApp with Parameter %1 failed with: %3", v_restart.ToString, ex.Message))
        End Try

        If v_restart AndAlso isRestart = False Then
            'Restart
            While _Splash IsNot Nothing AndAlso _Splash.Dispatcher.HasShutdownFinished = False
                Debug.Print("Tick")
            End While
            Process.Start(Application.ResourceAssembly.Location)
            Environment.Exit(0)
        Else
                Environment.Exit(0)
        End If
    End Sub

    Private Sub Application_DispatcherUnhandledException(sender As Object, e As DispatcherUnhandledExceptionEventArgs) Handles Me.DispatcherUnhandledException
        '#If Not DEBUG Then
        'Solves an issue with Avalon Dock see:
        'http://stackoverflow.com/questions/37834945/unhandled-system-componentmodel-win32exception-when-using-avalondock-2-0
        'https://avalondock.codeplex.com/workitem/17653
        'Soluition is not beautiful waiting for bug fix.
        If e.Exception.Source = "WindowsBase" Then
            cls_logger.v_logger.Info("WindowsBase Error detected AVALON")
            e.Handled = True
            Exit Sub
        End If
        MessageBox.Show("Ein Fehler ist aufgetreten: " & e.Exception.ToString & Environment.NewLine & "Anwendung wird beendet", "Fehler", MessageBoxButton.OK, MessageBoxImage.Error)
        cls_logger.v_logger.Error("Dispatcher:" & e.Exception.ToString) 'Create a Log in the Log File
        'Application.Current.Shutdown()
        Environment.Exit(99999)
    End Sub

    Private Sub Application_Exit(sender As Object, e As ExitEventArgs) Handles Me.[Exit]
        'When Application exits cleanup the viewmodel:
        HHHB3.ViewModel.ViewModelLocator.Cleanup()
        cls_logger.v_logger.Info("Viewmodel Cleanup done")
    End Sub
End Class