﻿Imports System.Collections.ObjectModel
Imports MahApps.Metro.Controls.Dialogs

''' <summary>
''' Klasse handelt den Umgang mit XML Dateien
''' </summary>
''' <remarks></remarks>
Public Class cls_xml_handler

    Public Enum e_status
        ok
        nok
    End Enum

    ''' <summary>
    ''' Erzeugt eine neue XML Konfigurations Datei unter dem angegeben Pfad
    ''' </summary>
    ''' <param name="v_filename">Name der neuen Datei</param>
    ''' <param name="v_pfad">Voller Ablagepfad muss mit \ enden!!</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function create_new_xml_config_file(v_filename As String, v_pfad As String, v_dbs As ObservableCollection(Of cls_lastdb)) As e_status
        On Error GoTo Fehlerbehandlung
        If v_filename = "" Then Return e_status.nok
        If v_pfad = "" Then Return e_status.nok

        ' Auswahl einer Kodierungsart für die Zeichenablage 
        Dim enc As New System.Text.UTF8Encoding

        ' XmlTextWriter-Objekt für unsere Ausgabedatei erzeugen: 
        Dim XMLobj As Xml.XmlTextWriter = New Xml.XmlTextWriter(v_pfad & v_filename, enc)

        With XMLobj
            ' Formatierung: 4er-Einzüge verwenden 
            .Formatting = Xml.Formatting.Indented
            .Indentation = 4

            ' Beginnen
            .WriteStartDocument()

            ' element Databases anlegen (Der Container für alle im System registrierten Datenbanken)
            .WriteStartElement("Databases")

            'Das element Database besteht aus Element "Name" und "Pfad" und "Pinned"
            'Zuerst die gepinnten:
            For Each v_db As cls_lastdb In v_dbs
                .WriteStartElement("Database")
                .WriteAttributeString("DBName", v_db.dbname)
                .WriteAttributeString("Path", v_db.pfad)
                .WriteAttributeString("Pinned", v_db.pinned.ToString)
                .WriteAttributeString("Add_Date", v_db.add_date.ToString)
                .WriteAttributeString("Sorter", v_db.sorter.ToString)
                .WriteEndElement()
            Next

            .WriteEndElement() ' </Personen> 

            ' ... und schließen das XML-Dokument (und die Datei) 
            .Close() ' Document 

        End With
        Return e_status.ok
        Exit Function
Fehlerbehandlung:
        Return e_status.nok

    End Function

    ''' <summary>
    ''' Liest das gegebene Config File aus sucht nach dem Wurzelelement "Databases" und gibt eine Liste
    ''' der Klasse db_lvwitem zurück
    ''' </summary>
    ''' <param name="v_file">Komplette Pfadangabe zur Config Datei</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function read_config_file(v_file As String) As List(Of cls_lastdb)

        'Temporäre Listenvariable
        Dim v_temp_lvwlist As New List(Of cls_lastdb)

        If v_file = "" Then Return Nothing

        Dim v_xmlfile As New Xml.XmlDocument

        Try
            With v_xmlfile
                .Load(v_file)
                'Existiert das Wurzelelement Databases?
                If Not .DocumentElement.Name = "Databases" Then Return Nothing
                'Gibt es Kinder?
                If Not .DocumentElement.HasChildNodes Then Return Nothing

                For i As Integer = 0 To .DocumentElement.ChildNodes.Count - 1
                    Debug.Print(.DocumentElement.ChildNodes(i).Name())
                    If .DocumentElement.ChildNodes(i).Name = "Database" Then
                        'Eine Datenbank gefunden Kinder prüfen:
                        Dim v_temp_name As String = ""
                        Dim v_temp_path As String = ""
                        Dim v_temp_pinned As Boolean = False
                        For z As Integer = 0 To .DocumentElement.ChildNodes(i).Attributes.Count - 1
                            Select Case .DocumentElement.ChildNodes(i).Attributes(z).Name
                                'Case "Name"
                                '    v_temp_name = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                Case "Path"
                                    v_temp_path = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                Case "Pinned"
                                    Dim v_temp_pinned_string As String = .DocumentElement.ChildNodes(i).Attributes(z).InnerText.ToString
                                    If v_temp_pinned_string = "True" Then
                                        v_temp_pinned = True
                                    Else
                                        v_temp_pinned = False
                                    End If
                            End Select
                        Next

                        v_temp_lvwlist.Add(New cls_lastdb() With {.pfad = v_temp_path, .pinned = v_temp_pinned})
                        v_temp_name = ""
                        v_temp_path = ""
                        v_temp_pinned = False

                    End If
                Next
            End With

            'Die Daten sind nun in der Temporären Liste:
            Return v_temp_lvwlist

        Catch ex As Exception
            Dim v_mainwindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow)
            v_mainwindow.ShowMessageAsync("Fehler", ex.Message, MessageDialogStyle.Affirmative)
            Return Nothing
        End Try
    End Function

End Class
