﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema
Imports System.Collections.Specialized
Imports System
Imports HHHB3.HHHB3

Public Class folder_v01
    Inherits model_base

    'Interne Variablen
    Dim v_fol_id As Integer
    Dim v_bezeichnung As String
    Dim v_color As String = "#000000"
    Dim v_aktiv As Nullable(Of Boolean) = True

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "bezeichnung"
                    If String.IsNullOrEmpty(bezeichnung) Then result = "Bezeichnung darf nicht leer sein"
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            Debug.Print("v_haserrors: " & v_haserrors.ToString)
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property fol_id As Integer
        Get
            Return v_fol_id
        End Get
        Set(value As Integer)
            If v_fol_id <> value Then
                v_fol_id = value
                NotifyPropertyChanged("fol_id")
            End If
        End Set
    End Property

    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If v_bezeichnung <> value Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
            End If
        End Set
    End Property

    Public Property color As String
        Get
            Return v_color
        End Get
        Set(value As String)
            If v_color <> value Then
                v_color = value
                NotifyPropertyChanged("color")
            End If
        End Set
    End Property

    Public Property aktiv As Nullable(Of Boolean)
        Get
            Return v_aktiv
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_aktiv.Equals(value) Then
                v_aktiv = value
                NotifyPropertyChanged("aktiv")
            End If
        End Set
    End Property
#End Region

#Region "Navigation Properties"
    Public Overridable Property nv_folder_2_konto As New ObservableCollection(Of konto_v01)

    <NotMapped>
    Public Property nv_folder_2_aktiv_kontos As ListCollectionView

#End Region

#Region "Extension"
    ''' <summary>
    ''' Property offers a SolidColorBrush out of the String HTML Color to change Foreground Color of an Item.
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property solid_color() As SolidColorBrush
        Get
            If color Is Nothing Then
                Return ColorBrushFromResources("IdealForegroundColorBrush")
            Else
                Try
                    Dim v_col As System.Drawing.Color = System.Drawing.ColorTranslator.FromHtml(color)
                    Dim v_color As Color = System.Windows.Media.Color.FromArgb(v_col.A, v_col.R, v_col.G, v_col.B)
                    Return New SolidColorBrush(v_color)
                Catch ex As Exception
                    Return ColorBrushFromResources("IdealForegroundColorBrush")
                End Try
            End If
        End Get
    End Property

    Public ReadOnly Property symbol_ever() As Byte()
        Get
            Return ResourceDictionaryImagetoByte("Folder")
        End Get
    End Property

    ''' <summary>
    ''' Calculated the totalamount of a folder within a given timerange
    ''' </summary>
    ''' <param name="v_begin">From Date</param>
    ''' <param name="v_end">To Date</param>
    ''' <returns>Totalamount of the folder</returns>
    Public Function TotalamountTimeFrame(v_begin As Date, v_end As Date) As Decimal
        If v_begin = Nothing OrElse v_end = Nothing Then Return 0
        'Total amount of the account
        Dim v_totamount As Decimal = 0
        For Each v_konto As konto_v01 In nv_folder_2_konto
            v_totamount += v_konto.TotalamountTimeFrame(v_begin, v_end, True)
        Next
        Return v_totamount
    End Function

    Dim Filters As New Filters.all_filters

    Sub New()
        _Collections = New INotifyCollectionChanged() {nv_folder_2_konto, nv_folder_2_aktiv_kontos}

        nv_folder_2_aktiv_kontos = New ListCollectionView(nv_folder_2_konto) With {
                .Filter = New Predicate(Of Object)(AddressOf Filters.ObjAktiv)
            }

    End Sub

    Public Sub refresh_konto_filter()
        If Not nv_folder_2_aktiv_kontos Is Nothing Then
            If Not nv_folder_2_aktiv_kontos.MoveCurrentToNext Then nv_folder_2_aktiv_kontos.MoveCurrentToPrevious()
            If Not nv_folder_2_aktiv_kontos.IsAddingNew Or nv_folder_2_aktiv_kontos.IsEditingItem Then nv_folder_2_aktiv_kontos.Filter = New Predicate(Of Object)(AddressOf Filters.ObjAktiv)
        End If
    End Sub

    ''' <summary>
    ''' Property for Archiv to define the String Type of the Object
    ''' </summary>
    ''' <returns></returns>
    <NotMapped>
    Public ReadOnly Property object_type As String
        Get
            Return "Ordner"
        End Get
    End Property
#End Region
End Class
