﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema
Imports System.Collections.Specialized
Imports System.Text

Public Class transactions_v01
    Inherits model_base
    Implements INotifyCollectionChanged

    'Hint: the totamount Property was formerly used in HHHB2 to display the entire amount of a transaction.
    'in HHHB3 the mathematical correct amount of an transaction is calculated "on the fly" through a sum action ob the nv_trans_t2k property
    'In hhhb2 there was a performance problem with this => review if same happens in hhhb3

    'Interne Variablen
    Dim v_trans_id As Integer
    Dim v_totamount As Decimal = 0D
    Dim v_comment As String
    Dim v_datum As Date = Date.Now
    Dim v_konto_id As Integer
    Dim v_to_konto_id As Nullable(Of Integer)
    Dim v_transtyp_id As Integer
    Dim v_transstatus_id As Nullable(Of Integer)
    Dim v_receiver_id As Nullable(Of Integer)
    Dim v_person_id As Nullable(Of Integer)

    Dim v_journal_id As Nullable(Of Integer)

    Sub New()
        Constructor_Helper()
    End Sub

    ''' <summary>
    ''' Use this constructor if you want to create a transaction from a transaction_repeat
    ''' </summary>
    ''' <param name="transaction_repeat"></param>
    Sub New(transaction_repeat As transactions_repeat_v01)
        With transaction_repeat
            comment = .comment
            datum = .faelligkeit
            konto_id = .konto_id
            to_konto_id = .to_konto_id
            transtyp_id = .transtyp_id
            transstatus_id = .transstatus_id
            receiver_id = .receiver_id
            person_id = .person_id
            'Copy categories:
            For Each v_tt2kat As transrep_2_kat In .categories_amount
                nv_trans_t2k.Add(New trans_2_kat With {.amount = v_tt2kat.amount, .category_id = v_tt2kat.category_id})
            Next
            TotAmountCalc()
            'Copy tags:
            For Each v_tag As transrep_tag_v01 In .tags
                nv_trans_transtag.Add(New trans_tag_v01 With {.tagid = v_tag.tagid})
            Next
        End With
        Constructor_Helper()
    End Sub

    Private Sub Constructor_Helper()
        _Collections = New INotifyCollectionChanged() {nv_trans_t2k, nv_trans_2_doc, nv_trans_transtag}
        'add CollectionChanged event to collections
        _Collections.ForEach(Sub(c) AddHandler c.CollectionChanged, AddressOf Collection_CollectionChanged)
    End Sub



    Public Sub Collection_CollectionChanged(sender As Object, e As NotifyCollectionChangedEventArgs)
        'Wenn eine der Navigation Properties geändert wird die in Frage kommenden Sonderproperties notifien
        NotifyPropertyChanged("real_totamount")
        NotifyPropertyChanged("totamount")
        NotifyPropertyChanged("real_totamount_str")
        NotifyPropertyChanged("neutral_totamount_str")
        NotifyPropertyChanged("list_categories")
        NotifyPropertyChanged("single_category_marker")
        NotifyPropertyChanged("tags_str")
    End Sub

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "datum"
                    If datum = Nothing Then result = "Datum darf nicht leer sein"
                Case "konto_id"
                    If konto_id = Nothing Then result = "Konto darf nicht leer sein"
                Case "to_konto_id"
                    If Not nv_transtyp_trans_back Is Nothing Then
                        If nv_transtyp_trans_back.internal_name = "transfer" Then
                            'Bei einer Transferbuchung muss das Zielkonto definiert sein:
                            If to_konto_id Is Nothing Then result = "Bei einer Transferbuchung muss ein Zielkonto angegeben werden"
                        End If
                    End If
                Case "transtyp_id"
                    If transtyp_id = Nothing Then result = "Typ darf nicht leer sein"
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property trans_id As Integer
        Get
            Return v_trans_id
        End Get
        Set(value As Integer)
            If v_trans_id <> value Then
                v_trans_id = value
                NotifyPropertyChanged("trans_id")
            End If
        End Set
    End Property

    Public Property totamount As Decimal
        Get
            Return v_totamount
        End Get
        Set(value As Decimal)
            If v_totamount <> value Then
                v_totamount = value
                NotifyPropertyChanged("totamount")
                NotifyPropertyChanged("real_totamount")
            End If
        End Set
    End Property

    Public Property journal_id As Nullable(Of Integer)
        Get
            Return v_journal_id
        End Get
        Set(value As Nullable(Of Integer))
            If v_journal_id <> value Then
                v_journal_id = value
                NotifyPropertyChanged("journal_id")
                NotifyPropertyChanged("link_status_image")
                NotifyPropertyChanged("konto_change_allowed")
            End If
        End Set
    End Property

    Public Property comment As String
        Get
            Return v_comment
        End Get
        Set(value As String)
            If v_comment <> value Then
                v_comment = value
                NotifyPropertyChanged("comment")
                'Necessary to notify that short_comment property has also changed
                NotifyPropertyChanged("short_comment")
            End If
        End Set
    End Property

    Public Property datum As Date
        Get
            Return v_datum
        End Get
        Set(value As Date)
            If v_datum <> value Then
                v_datum = value
                NotifyPropertyChanged("datum")
            End If
        End Set
    End Property

    Public Property konto_id As Integer
        Get
            Return v_konto_id
        End Get
        Set(value As Integer)
            If v_konto_id <> value Then
                v_konto_id = value
                NotifyPropertyChanged("konto_id")
            End If
        End Set
    End Property

    Public Property to_konto_id As Nullable(Of Integer)
        Get
            Return v_to_konto_id
        End Get
        Set(value As Nullable(Of Integer))
            'Change to_konto_id always because of the fact that if user makes a transfer and changes back to another type konto_id and also to_konto_id can be assigned => failure
            ' If v_to_konto_id Is Nothing Or v_to_konto_id <> value Then
            v_to_konto_id = value
            NotifyPropertyChanged("to_konto_id")
            ' End If
        End Set
    End Property

    Public Property transtyp_id As Integer
        Get
            Return v_transtyp_id
        End Get
        Set(value As Integer)
            If v_transtyp_id <> value Then
                v_transtyp_id = value
                NotifyPropertyChanged("transtyp_id")
            End If
        End Set
    End Property

    Public Property transstatus_id As Nullable(Of Integer)
        Get
            Return v_transstatus_id
        End Get
        Set(value As Nullable(Of Integer))
            If v_transstatus_id Is Nothing Or v_transstatus_id <> value Then
                If value = 0 Then
                    v_transstatus_id = Nothing
                Else
                    v_transstatus_id = value
                End If
                NotifyPropertyChanged("transstatus_id")
            End If
        End Set
    End Property

    Public Property receiver_id As Nullable(Of Integer)
        Get
            Return v_receiver_id
        End Get
        Set(value As Nullable(Of Integer))
            If v_receiver_id Is Nothing Or v_receiver_id <> value Then
                If value = 0 Then
                    v_receiver_id = Nothing
                Else
                    v_receiver_id = value
                End If
                NotifyPropertyChanged("receiver_id")
            End If
        End Set
    End Property

    Public Property person_id As Nullable(Of Integer)
        Get
            Return v_person_id
        End Get
        Set(value As Nullable(Of Integer))
            If v_person_id Is Nothing Or v_person_id <> value Then
                If value = 0 Then
                    v_person_id = Nothing
                Else
                    v_person_id = value
                End If
                NotifyPropertyChanged("person_id")
            End If
        End Set
    End Property
#End Region

#Region "Navigation Properties"
    Dim v_nv_kto_2_trans_back As konto_v01
    Dim v_nv_kto_2_trans_transfer_back As konto_v01
    Dim v_nv_transtyp_trans_back As trans_typ_v01
    Dim v_nv_transstatus_trans_back As trans_status_v01
    Dim v_nv_receiver_trans_back As receiver_v01
    Dim v_nv_person_trans_back As person_v01


    <ForeignKey("konto_id")>
    Public Property nv_kto_2_trans_back As konto_v01
        Get
            Return v_nv_kto_2_trans_back
        End Get
        Set(value As konto_v01)
            If v_nv_kto_2_trans_back IsNot value Then
                v_nv_kto_2_trans_back = value
                NotifyPropertyChanged("nv_kto_2_trans_back")
            End If
        End Set
    End Property
    <ForeignKey("to_konto_id")>
    Public Property nv_kto_2_trans_transfer_back As konto_v01
        Get
            Return v_nv_kto_2_trans_transfer_back
        End Get
        Set(value As konto_v01)
            If v_nv_kto_2_trans_transfer_back IsNot value Then
                v_nv_kto_2_trans_transfer_back = value
                NotifyPropertyChanged("nv_kto_2_trans_transfer_back")
            End If
        End Set
    End Property
    <ForeignKey("transtyp_id")>
    Public Property nv_transtyp_trans_back As trans_typ_v01
        Get
            Return v_nv_transtyp_trans_back
        End Get
        Set(value As trans_typ_v01)
            If v_nv_transtyp_trans_back IsNot value Then
                v_nv_transtyp_trans_back = value
                NotifyPropertyChanged("nv_transtyp_trans_back")
            End If
        End Set
    End Property
    <ForeignKey("transstatus_id")>
    Public Property nv_transstatus_trans_back As trans_status_v01
        Get
            Return v_nv_transstatus_trans_back
        End Get
        Set(value As trans_status_v01)
            If v_nv_transstatus_trans_back IsNot value Then
                v_nv_transstatus_trans_back = value
                NotifyPropertyChanged("nv_transstatus_trans_back")
            End If
        End Set
    End Property
    Public Overridable Property nv_trans_t2k As New ObservableCollection(Of trans_2_kat)
    <ForeignKey("receiver_id")>
    Public Property nv_receiver_trans_back As receiver_v01
        Get
            Return v_nv_receiver_trans_back
        End Get
        Set(value As receiver_v01)
            If v_nv_receiver_trans_back IsNot value Then
                v_nv_receiver_trans_back = value
                NotifyPropertyChanged("nv_receiver_trans_back")
            End If
        End Set
    End Property
    <ForeignKey("person_id")>
    Public Property nv_person_trans_back As person_v01
        Get
            Return v_nv_person_trans_back
        End Get
        Set(value As person_v01)
            If v_nv_person_trans_back IsNot value Then
                v_nv_person_trans_back = value
                NotifyPropertyChanged("nv_person_trans_back")
            End If
        End Set
    End Property
    Public Overridable Property nv_trans_2_doc As New ObservableCollection(Of documents_v01)
    Public Overridable Property nv_trans_transtag As New ObservableCollection(Of trans_tag_v01)

    Dim v_nv_journal As bookjournal_entry_v01

    'The 0or1 to 0or1 property has to be defined in FluentAPI (see hhhb3_model class)
    Public Overridable Property nv_journal As bookjournal_entry_v01
        Get
            Return v_nv_journal
        End Get
        Set(value As bookjournal_entry_v01)
            If v_nv_journal IsNot value Then
                v_nv_journal = value
                NotifyPropertyChanged("nv_journal")
                NotifyPropertyChanged("link_status_image")
                NotifyPropertyChanged("konto_change_allowed")
            End If
        End Set
    End Property

#End Region

#Region "Extension"

    ''' <summary>
    ''' Returns all associated tags in a single string.
    ''' String empty when no tags are associated
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property tags_str As String
        Get
            If nv_trans_transtag Is Nothing OrElse nv_trans_transtag.Count = 0 Then Return String.Empty
            Dim v_strbuilder As New StringBuilder
            For Each v_tag As trans_tag_v01 In nv_trans_transtag
                If Not v_tag.nv_tag_tt_back Is Nothing Then v_strbuilder.Append(v_tag.nv_tag_tt_back.bezeichnung & "; ")
            Next
            Dim v_realstring As String = v_strbuilder.ToString
            If v_realstring.EndsWith("; ") Then v_realstring = v_realstring.Substring(0, v_realstring.Length - 2)
            Return v_realstring
        End Get
    End Property

    ''' <summary>
    ''' Gives back an Image depending on the link stage to a bookjournal entry:
    ''' OK => is linked to a transaction
    ''' NOK => is not linked
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property link_status_image As Byte()
        Get
            If nv_journal Is Nothing Then
                Return ResourceDictionaryImagetoByte("link_nok")
            Else
                Return ResourceDictionaryImagetoByte("link_ok")
            End If
        End Get
    End Property

    Public ReadOnly Property link_status_str As String
        Get
            If nv_journal Is Nothing Then
                Return "not linked"
            Else
                Return "linked"
            End If
        End Get
    End Property

    ''' <summary>
    ''' Decides if the related Konto of an Transaction can be changed or not.
    ''' Konto Change is usually allowed but not in case of an existing link to a journal entry
    ''' </summary>
    ''' <returns>True... is unlinked
    ''' False.... is linked and change not allowed</returns>
    Public ReadOnly Property konto_change_allowed As Boolean
        Get
            If nv_journal Is Nothing Then
                Return True
            Else
                Return False
            End If
        End Get
    End Property


    <NotMapped>
    Public Property Kategorien As ListCollectionView = New ListCollectionView(nv_trans_t2k)

#Region "Special Properties"

    ''' <summary>
    ''' Property gibt den mathematisch korrekten Wert einer Buchung an
    ''' Es ist definiert wie folgt:
    ''' Eingang = +
    ''' Ausgang = -
    ''' Transfer = 0
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property real_totamount As Decimal
        Get
            If nv_transtyp_trans_back Is Nothing Then Return 0
            'durch den Multiplikator ist ein Select Case nicht mehr notwendig:
            Return totamount * nv_transtyp_trans_back.multiplier
        End Get
    End Property

    Public ReadOnly Property real_totamount_str As String
        Get
            Return FormatCurrency(real_totamount, 2,,, TriState.True)
        End Get
    End Property

    'Public ReadOnly Property neutral_totamount As Decimal
    '    Get
    '        Return totamount 'nv_trans_t2k.Sum(Function(p) p.amount)
    '    End Get
    'End Property

    Public Function TotAmountCalc() As Boolean
        totamount = nv_trans_t2k.Sum(Function(p) p.amount)
        NotifyPropertyChanged("totamount")
        Call notify_value_changed()
        Return True
    End Function

    Public ReadOnly Property neutral_totamount_str As String
        Get
            Return FormatCurrency(totamount, 2,,, TriState.True)
        End Get
    End Property

    Public Sub notify_value_changed()
        NotifyPropertyChanged("real_totamount")
        NotifyPropertyChanged("real_totamount_str")
        NotifyPropertyChanged("neutral_totamount_str")
    End Sub

    ''' <summary>
    ''' This function solves the problem of transfer transactions which can be either positive or negative depending on which konto is looking on.
    ''' 'Requires the Account ID to decide if a transfer is positive or negative
    ''' </summary>
    ''' <returns></returns>
    Public Function get_transfer_totamount(v_check_kto_id As Integer) As Decimal
        If v_check_kto_id = Nothing Then Return 0

        Debug.Print("Transid: " & trans_id.ToString & " Totamount: " & totamount.ToString & " and real_totamount: " & real_totamount.ToString)
        Try
            If nv_transtyp_trans_back.multiplier = 0 Then
                'it's a transfer
                If to_konto_id IsNot Nothing AndAlso to_konto_id = v_check_kto_id Then
                    Return totamount
                Else
                    Return totamount * -1
                End If
            Else
                Return real_totamount
            End If
        Catch ex As Exception
            Debug.Print(ex.Message)
            MessageBox.Show("Fehler bei geT_transfer_totamount " & ex.Message)
            Return 0
        End Try
    End Function


    Public ReadOnly Property short_comment As String
        Get
            If comment = "" Then
                Return ""
            Else
                'Trim after 40 signs and remove Line Feeds
                Dim v_temp As String = comment.Replace(Environment.NewLine, " ")
                If v_temp.Length < 40 Then
                    Return v_temp
                Else
                    Return v_temp.Substring(0, 39) & "..."
                End If
            End If
        End Get
    End Property

    Public ReadOnly Property comment_to_copy As String
        Get
            If comment = String.Empty Then
                Return String.Empty
            Else
                Return comment.Replace(Environment.NewLine, " ")
            End If
        End Get
    End Property

    ''' <summary>
    ''' Summarizes the used Categories to an String for displaying in Datagrid
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property list_categories As String
        Get
            If nv_trans_t2k.Count = 0 Then
                Return ""
            Else
                Dim v_cats As New StringBuilder
                For Each v_item As trans_2_kat In nv_trans_t2k
                    If Not v_item.nv_cat_trans2kat_back Is Nothing Then
                        v_cats.AppendLine(v_item.nv_cat_trans2kat_back.bezeichnung)
                    End If
                Next
                Return v_cats.ToString
            End If
        End Get
    End Property

    Public ReadOnly Property single_category_marker As String
        Get
            If nv_trans_t2k.Count = 0 Then
                Return ""
            ElseIf nv_trans_t2k.Count = 1 Then
                Dim v_t2k As trans_2_kat = DirectCast(nv_trans_t2k.First, trans_2_kat)
                If v_t2k.nv_cat_trans2kat_back Is Nothing Then
                    Return String.Empty
                Else
                    Return v_t2k.nv_cat_trans2kat_back.fullname
                End If
            Else
                Return "Mehrfach - " & nv_trans_t2k.Count.ToString()
            End If
        End Get
    End Property

#End Region
#End Region

End Class
